package com.example.jpa;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;
import javax.persistence.Query;
import javax.sql.DataSource;

/**
 * Session Bean testing the performance of several ways to call SELECT *
 */
@Stateless
public class JPAPerfTestBean implements JPAPerfTestBeanLocal {

	private static Logger LOG =  Logger.getLogger(JPAPerfTestBean.class.getName());
	
	EntityManager em = null;

	@PersistenceUnit(unitName="testpu")
	EntityManagerFactory emf;
	
	@Resource(mappedName="jdbc/__default") 
    DataSource dataSource;

	private Connection jdbcConnection;
	
	Query queryJPA = null;
	Query queryEJB = null;
	PreparedStatement stmt1 = null;
	
	@PostConstruct 
	public void init() 
	{
	  em = emf.createEntityManager();
	  try {
		jdbcConnection = dataSource.getConnection();
	  } catch (SQLException e) {
		e.printStackTrace();
	  }
      prepareQueries();
	}
	
	@PreDestroy public void destroy()
	{
	  em.close();
	}
	
    public void prepareQueries()
    {
    	String statement = "SELECT * from customer";
    	queryJPA = em.createNativeQuery(statement);
    	queryJPA.setHint("eclipselink.jdbc.fetch-size", 500);
    	queryJPA.setHint("eclipselink.jdbc.read-only", true); 
    	try {
			stmt1 = jdbcConnection.prepareStatement(statement/*, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY*/);
		} catch (SQLException e) {
			e.printStackTrace();
		}
        queryEJB = em.createNativeQuery("SELECT C.* FROM CUSTOMER C", Customer.class);
    	queryEJB.setHint("eclipselink.jdbc.fetch-size", 500);
    	queryEJB.setHint("eclipselink.jdbc.read-only", true); 

    }
    
    /**
     * This is expected to be the fastest still. It does offer the worst usability, though.
     */
    public void doQueryJDBC(int count)
    {
    	LOG.info("start JDBC");
    	for (int i=0; i<count; i++)
    	{
    		try {
				ResultSet rs = stmt1.executeQuery();
				while (rs.next())
				{
					int param = rs.getInt(1); // let's hope the optimizer isn't smart enough for this.
				}
			} catch (SQLException e) {
				e.printStackTrace();
			}
    	}
    	LOG.info("finish JDBC");
    }
    
    /**
     * Most coder-friendly version, as we're getting O/R-mapped Customer-Objects.
     * Performance is worst though but that's OK as long as it isn't catastrophic.
     */
    public void doQueryEJB(int count)
    {
    	LOG.info("start EJB");
        for (int i=0; i<count; i++)
    	{
    		List l = queryEJB.getResultList();
    		for (Customer c: (List<Customer>)l)
    		{
    			String n = c.getName(); // let's hope the optimizer isn't smart enough for this.
    		}
    	}
    	LOG.info("finish EJB");
    }    

    /**
     * Using a native query should be almost as the JDBC version above, but it isn't. 
     * It's almost as slow as the EJB version.
     * There seems to be some kind of O/R mapping involved (to an Object[]), which might cause the performance ditch.
     */
    public void doQueryJPA(int count)
    {
    	LOG.info("start JPA");
    	for (int i=0; i<count; i++)
    	{
    		List l = queryJPA.getResultList();
    		for (Object[] o: (List<Object[]>)l)
    		{
    			String s = (String)o[1]; // let's hope the optimizer isn't smart enough for this.
    		}
    	}
    	LOG.info("finish JPA");
    }
}
